<?php

namespace Imagine;

class Polygon {

    static $mode = "rad";

    const sqrt_2 = 1.41421356237;

    public static function draw(resource $image, $colors, $vertexcount, int $x, int $y, $size, $angle = 0) : array
    {
        $coord = self::polygonCoord($vertexcount, $x, $y, $size, $angle);
        $data = array_values($coord);

        if ( $colors['brush'] ) {
            \imagefilledpolygon($image, $data, count($data) / 2, $colors['brush']);
        }

        return [
            "return" => \imagepolygon($image, $data, count($data) / 2, $colors['fore']),
            "points" => $data,
            "coord"  => $coord
        ];
    }

    public static function polygonCoord($vertexcount, $x, $y, $size, $angle = 0) : array
    {
        $n   = 0;
        $r   = $size / 2;
        $pi2 = M_PI * 2;

        $points = [];
        while ( $n < $vertexcount ) {
            $points["x$n"] = $r * cos($pi2 * $n / $vertexcount + $angle) + $x;
            $points["y$n"] = $r * sin($pi2 * $n / $vertexcount + $angle) + $y;
            $n++;
        }

        return $points;
    }

    public static function rotatePoint($theta, int $x, int $y, int $px, int $py) : array
    {
        $cos = cos($theta);
        $sin = sin($theta);

        return [
            'x' => $cos * ($x - $px) - $sin * ($y - $py) + $px,
            'y' => $sin * ($x - $px) + $cos * ($y - $py) + $py
        ];
    }

    public function dist(int $x1, int $y1, int $x2, int $y2) : float
    {
		return sqrt( ($x1 - $x2) * ($x1 - $x2) + ($y1 - $y2) * ($y1 - $y2) );
    }

    /*
	** Calculate the angle between 2 points, where Xc,Yc is the center of a circle
	** and x,y is a point on its circumference. All angles are relative to
	** the 3 O'Clock position. Result returned in radians
	*/
	public function angle($xc, $yc, $x1, $y1) : float
	{
        # calculate distance between two points
		$d = $this->distBetween($xc, $yc, $x1, $y1);

		if ($d != 0)
			if ( asin( ($y1-$yc)/$d ) >= 0 )
				$a1 = acos( ($x1-$xc)/$d );
			else
				$a1 = 2 * pi() - acos( ($x1-$xc)/$d );
		else
			$a1 = 0;

		return $a1;
	}

    public static function drawRectangle(resource $image, int $x1, int $y1, int $x2, int $y2, array $colors) : bool
    {
        if ( $colors['brush'] ) {
            \imagefilledrectangle($image, $x1, $y1, $x2, $y2, $colors['brush']);
        }

        return \imagerectangle ( $image , $x1 , $y1 , $x2 , $y2 , $colors['fore'] );
    }

    public static function drawIrregular(resource $image, array $colors, array $points) : array
    {
        $data = array_values($points);

        if ( $colors['brush'] ) {
            \imagefilledpolygon($image, $data, count($data) / 2, $colors['brush']);
        }

        return [
            "return" => \imagepolygon($image, $data, count($data) / 2, $colors['fore']),
            "points" => $data
        ];
    }

    public static function drawEllipse(resource $image, array $colors, int $posX, int $posY, int $width, int $height) : bool
    {

        if ( $colors['brush'] ) {
            \imagefilledellipse($image, $posX, $posY, $width, $height, $colors['brush']);
        }

        return \imageellipse($image, $posX, $posY, $width, $height, $colors['fore']);
    }

}
